
/*
  Copyright (c) 2009 Dell Inc. by Narendra K <Narendra_K@dell.com>

  libnetdevname library for network device name mapping

  functions to map alias name of a network  interface to it's eth name and 
  given an eth name provide a list of  aliases
       0 - success 
      -1 - failure 

  Permission is hereby granted, free of charge, to any person
  obtaining a copy of this software and associated documentation
  files (the "Software"), to deal in the Software without
  restriction, including without limitation the rights to use,
  copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be
  included in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
  OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
  OTHER DEALINGS IN THE SOFTWARE.
*/

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <net/if.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <glob.h>

#include "netdevname.h"

#define SYSFS_ALIAS_PATH	"/sys/class/net/%s/device/smbiosname"
#define SYSFS_ALIAS_PATTERN	"/sys/class/net/eth*/device/smbiosname"
#define BUF_SIZE		sizeof(SYSFS_ALIAS_PATH) + IFNAMSIZ

int netdev_kernelname_to_alias(char *kernelname, struct alias **aliasptr) {
	
	struct stat buf;
	char sysfs_alias_path[BUF_SIZE];
	off_t len;
	int fd;
	int ret;

	snprintf(sysfs_alias_path, BUF_SIZE, SYSFS_ALIAS_PATH, kernelname);

	/* if smbiosname attribute does not exist for the given interface, abort */
	if(stat(sysfs_alias_path, &buf) == -1) {
		perror("netdev_kernelname_to_alias: stat failed");
		return -1;
	}

	len = buf.st_size;

	/* open, read and store the smbios string in struct alias */
	if ((fd = open(sysfs_alias_path, O_RDONLY)) < 0) {
		perror("netdev_kernelname_to_alias: open failed");
		return -2;
	}

	*aliasptr = malloc(sizeof(struct alias));
	memset((*aliasptr), 0, sizeof(struct alias));
	(*aliasptr)->name = malloc(len +1);
	memset((*aliasptr)->name, 0, len + 1);

	if ((ret = read(fd, (*aliasptr) -> name, len) < 0)) {
		perror("netdev_kernelname_to_alias: read failed");
		close(fd);
		free((*aliasptr)->name);
		free(*aliasptr);
	}

	/* indicate which of the aliases is descriptive, in this case it is smbios name 
 	 * of the interface. If the same interface has multiple aliases, then build the 
 	 * list and set is_descriptive to 1 for the relavant alis
 	 */ 
	(*aliasptr)->name[len] = '\0';
	(*aliasptr)-> is_descriptive = 1;
	(*aliasptr)-> next = NULL;

	return 0;

}

int netdev_alias_to_kernelname(char *alias, char *kernelname) {

	glob_t globbuf;
	struct stat buf;
	int i = 0;
	int fd;
	int ret;
	off_t len;
	char *name, *ptr;
	
	/* if alias name is of the form ethN, please return it as it is */
	if (!strncmp(alias, "eth", 3)) {
		strncpy(kernelname, alias, strlen(alias) + 1);
		return 0;
	}
	
	/* get a list of pathnames of the form /sys/class/net/ethN/device/smbiosname */
	if (glob(SYSFS_ALIAS_PATTERN, 0, NULL, &globbuf) != 0) {
		perror("netdev_alias_to_kernelname: glob failed");
		return -1;
	}
	
	/*
 	 *  read each file and compare the read string to the alias(Embedded NIC N) and 
 	 *  if a match is found return the corresponding ethN string
	 */
	for (i = 0; globbuf.gl_pathv[i]; i++) {
		if (stat(globbuf.gl_pathv[i], &buf) == -1) {
			perror("netdev_alias_to_kernelname: stat failed");
			return -1;
		}

		len = buf.st_size;
		
		if ((fd = open(globbuf.gl_pathv[i], O_RDONLY)) < 0) {
			perror("netdev_alias_to_kernelname: open failed");
			return -2;
		}	
		
		name = malloc(len +1);
		memset(name, 0, len + 1);
		
		if ((ret = read(fd, name, len) < 0)) {
			perror("netdev_alias_to_kernelname: read failed");
			free(name);
			close(fd);
			continue;
		}

		name[len] = '\0';
		
		if (strncmp(alias, name, strlen(alias)) == 0) {
			/* retreive the "ethN" string from /sys/class/net/ethN/device/smbiosname */
			if ((ptr = strstr(globbuf.gl_pathv[i], "eth"))) {
				while ( (*ptr) != '/') 
					*kernelname++ = *ptr++;
				*kernelname = '\0';
			}	
		}
		free(name);
	}
	return 0;
}

int free_netdev_alias_info (struct alias **ptr) {
	
	struct alias *tmp, *nextptr;
	
	/* check for the null */
	if (!ptr) {
		fprintf(stderr, "free_netdev_alias_info: cannot free null string\n");
		return -1;
	}

	/* run through the list of aliases and free the structures */
	tmp = *ptr;
	while (tmp->next) {
		nextptr = tmp->next;
		free(tmp->name);
		free(tmp);
		tmp = nextptr;
	}
	free(tmp->name);
	free(tmp);
}
